package br.com.mobits.sectionlist.adapter;

import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import br.com.mobits.sectionlist.util.SectionIndex;


public abstract class SectionAdapter extends BaseAdapter {

	public static final int SECTION_TYPE = 0;
	public static final int ITEM_TYPE = 1;
	
	abstract protected int getSectionCount();
	abstract protected int getItemCountForSection(int sectionIndex);
	abstract protected Object getItemForSection(int sectionIndex, int itemIndex);
	abstract protected long getItemIdForSection(int sectionIndex, int itemIndex);
	abstract protected View getSectionView(int sectionIndex, View convertView, ViewGroup parent);
	abstract protected View getSectionItemView(int sectionIndex, int itemIndex, View convertView, ViewGroup parent);
	
	@Override
	public final int getCount() {
		int sectionCount = getSectionCount();
		int itemCount = 0;
		
		for (int i = 0; i < sectionCount; i++) {
			itemCount++;
			
			itemCount += getItemCountForSection(i);
		}
		
		return itemCount;
	}
	
	@Override
	public final Object getItem(int index) {

		SectionIndex sectionIndex = getSectionIndex(index);
		
		if (sectionIndex != null) {
			if (sectionIndex.isItem())
				return getItemForSection(sectionIndex.getSection(), sectionIndex.getItem());
		}

		return null;
	}

	@Override
	public final long getItemId(int index) {
		
		SectionIndex sectionIndex = getSectionIndex(index);
		
		if (sectionIndex != null) {
			if (sectionIndex.isItem())
				return getItemIdForSection(sectionIndex.getSection(), sectionIndex.getItem());
		}
		
		return 0;
	}

	@Override
	public final int getViewTypeCount() {
		return 2;
	}
	

	@Override
	public final int getItemViewType(int index) {
		
		SectionIndex sectionIndex = getSectionIndex(index);
		
		if (sectionIndex != null) {
			if (sectionIndex.isSection())
				return SECTION_TYPE;
			
			if (sectionIndex.isItem())
				return ITEM_TYPE;
		}
		
		return IGNORE_ITEM_VIEW_TYPE;
	}
	
	@Override
	public boolean isEnabled(int index) {
		SectionIndex sectionIndex = getSectionIndex(index);
		
		if (sectionIndex != null) {
			if (sectionIndex.isSection())
				return false;
			
			if (sectionIndex.isItem())
				return true;
		}
		
		return super.isEnabled(index);
	}
	
	@Override
	public final View getView(int index, View convertView, ViewGroup parent) {
		
		SectionIndex sectionIndex = getSectionIndex(index);
		
		if (sectionIndex != null) {
			if (sectionIndex.isSection())
				return getSectionView(sectionIndex.getSection(), convertView, parent);
			
			if (sectionIndex.isItem())
				return getSectionItemView(sectionIndex.getSection(), sectionIndex.getItem(), convertView, parent); 
		}

		return null;
	}
	
	private SectionIndex getSectionIndex(int index) {
		if (index >= getCount())
			return null;
		
		int sectionCount = getSectionCount();
		int itemCount = 0;
		
		for (int i = 0; i < sectionCount; i++) {
			if (index == itemCount)
				return new SectionIndex(i);
			
			itemCount++;
			
			int itemCountForSection = getItemCountForSection(i);
			
			if (index < itemCount + itemCountForSection) {
				for (int j = 0; j < itemCountForSection; j++) {
					if (index == itemCount)
						return new SectionIndex(i, j);
					
					itemCount++;
				}
			}
			else {
				itemCount += itemCountForSection;
			}
		}

		return null;
	}
	
}
